#!/usr/bin/env python
#
# -------------------------------------------------------------------------------------
#
# Copyright (c) 2016 KGH
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#    - Redistributions of source code must retain the above copyright notice, this 
#    list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above copyright notice, 
#    this list of conditions and the following disclaimer in the documentation and/or 
#    other materials provided with the distribution.
#    - Neither the name of the author nor the names of its contributors may be used 
#    to endorse or promote products derived from this software without specific prior 
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
# SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
# DAMAGE.
#
# -------------------------------------------------------------------------------------
#
# This file is a basic Python plug-in for GIMP.
#
# It can be executed by selecting the menu option: 'Filters>Custom Font Labels'
#
# The first option will re-order the layers in the custom font file in the following order
#		space.png (or  .ping)
#		Capital letters (A-Z)
#		Lower case letters (a-z)
#		Numbers (0-9)
#		Basic maths characters
# 		Punctuation characters
#		Other keyboard characters
#		Special characters (Pound Sterling sign, Copyright Sign, Registered Trademark sign)
#			You can change this by altering lines 79 - 86 of the plugin
# The second option will throw up a message box listing all the characters (ie layers) defined in the custom font
# The third option will add .png to all the layers with one character names ie  A to A.png
# The fourth option will add .png to all the paths with one character names
# The fifth option creates new letter faces from an over-sized visible layer/s
# The sixth option removes non-font character layers from the image - apart from any layer name containing the word legend

# Version 0.01 Fixed adding .png to single letter layers and paths with special characters ie unicode characters
# Version 1.02 Adds an option to create a new face for a custom font from additional/decorative visible layers
# Version 1.03 Re-position space layer, if one exists, along with other letter layers and adds the option to remove non-font-character layers from the image
# Version 1.04 Trim and feather pixels from letter faces to remove halo effect

from gimpfu import *
from array import array
from gtk import gdk
import math
import random

w = gdk.screen_width()
h = gdk.screen_height()

# Some global colours we may need to use

black = gimpcolor.RGB(0, 0, 0)
white = gimpcolor.RGB(255, 255, 255)
darkgrey = gimpcolor.RGB(50, 50, 50)
midgrey = gimpcolor.RGB(125, 125, 125)
lightgrey = gimpcolor.RGB(170, 170, 170)
verylightgrey = gimpcolor.RGB(222, 222, 222)
red = gimpcolor.RGB(220,10,10)	


def do_stuff(image, layer, text_string, do_sort, do_layer_info, do_layer_names, do_path_names, do_new_letters, do_remove_non_letter_layers):


    # Store the GIMP's settings so they can be restored
    gimp.context_push()
    # Set start undo point
    pdb.gimp_image_undo_group_start(image)
	
    if do_remove_non_letter_layers:
        
        loop_ptr = 1
        num_layers, layer_ids = pdb.gimp_image_get_layers(image)
        loop = num_layers - loop_ptr
        layer_names_list = image.layers
	
        visi_loop, other_layers, first_letter_layer, has_a_space = 0, 0, -1, False
        do_not_remove = "legend"
        
        while visi_loop < num_layers:
            # check for layers to create new faces for
            visi_name = pdb.gimp_item_get_name(layer_names_list[visi_loop])
            visi_name = visi_name.decode('utf-8')
            if len(visi_name) == 1 or (len(visi_name) == 5 and ".png" in visi_name) or visi_name == "space.png" or do_not_remove.lower() in visi_name.lower():
			    # Re-position letter layers
                layer_names_list[visi_loop].set_offsets(0, 0)
            else:
			    # Remove other layers
                pdb.gimp_image_remove_layer(image, layer_names_list[visi_loop])
                pdb.gimp_image_resize_to_layers(image)
                    		
            visi_loop += 1
			
			
    if do_new_letters:
	
        im_width = image.width
        im_height = image.height
    
        loop_ptr = 1
        num_layers, layer_ids = pdb.gimp_image_get_layers(image)
        loop = num_layers - loop_ptr
        layer_names_list = image.layers
		
		
        visi_loop, other_layers, first_letter_layer, has_a_space = 0, 0, -1, False
        
        while visi_loop < num_layers:
            # check for layers to create new faces for
            visi_name = pdb.gimp_item_get_name(layer_names_list[visi_loop])
            visi_name = visi_name.decode('utf-8')
            if visi_name == "space.png" or visi_name == " .png": has_a_space = True
            if len(visi_name) == 1 or len(visi_name) == 5 and ".png" in visi_name:
                if first_letter_layer < 0: first_letter_layer = visi_loop
            else:
                pdb.gimp_image_select_item(image, 2, layer_names_list[visi_loop])
                is_empty = pdb.gimp_selection_is_empty(image)
                if is_empty == False: other_layers += 1	
                    		
            visi_loop += 1
			
        visi_loop = 0
        
        while visi_loop < num_layers and other_layers > 0:
            visi_name = pdb.gimp_item_get_name(layer_names_list[visi_loop])
            visi_name = visi_name.decode('utf-8')
            if visi_name == "space.png" or visi_name == " .png": 
                has_a_space = True
                space_loop = visi_loop
            if len(visi_name) == 1 or len(visi_name) == 5 and ".png" in visi_name:
                # turn off visibility of letter layers
                pdb.gimp_item_set_visible(layer_names_list[visi_loop],False)  
            visi_loop += 1
			
        if other_layers > 0:
	
            visi_loop = 0
            while visi_loop < num_layers:
                visi_name = pdb.gimp_item_get_name(layer_names_list[visi_loop])
                visi_name = visi_name.decode('utf-8')
                if len(visi_name) == 1 or len(visi_name) == 5 and ".png" in visi_name:
                    # move layer to random position within the confines of the image size
                    layer_width = layer_names_list[visi_loop].width
                    layer_height = layer_names_list[visi_loop].height
                    random.seed()
                    random_x = random.randrange(0, (im_width + 1) - layer_width)
                    random.seed()
                    random_y = random.randrange(0, (im_height + 1) - layer_height)
                    layer_names_list[visi_loop].set_offsets(random_x, random_y)
				
                    # select the contents of the layer
                    pdb.gimp_image_select_item(image, 2, layer_names_list[visi_loop])
                    pdb.gimp_drawable_edit_clear(layer_names_list[visi_loop])
				
                    # copy the contents of the visible layers into the selection
                    success = pdb.gimp_edit_copy_visible(image)
                    pdb.gimp_drawable_edit_clear(layer_names_list[visi_loop])
                    # paste the selection into the floating layer
                    floating_sel = pdb.gimp_edit_paste(layer_names_list[visi_loop],True)
                    pdb.gimp_floating_sel_anchor(floating_sel)
                    proc_trim_away(image, layer_names_list[visi_loop], 1)
				
			    	# Re-position letter layers
                    layer_names_list[visi_loop].set_offsets(0, 0)
				
                
                visi_loop += 1
				
                if has_a_space:
                    layer_names_list[space_loop].set_offsets(0, 0)
		
                # Deselect
                pdb.gimp_selection_none(image)
				
            visi_loop = 0
        
            while visi_loop < num_layers and other_layers > 0:
                visi_name = pdb.gimp_item_get_name(layer_names_list[visi_loop])
                visi_name = visi_name.decode('utf-8')
                if visi_loop <> first_letter_layer:
                    # turn off visibility of letter layers
                    pdb.gimp_item_set_visible(layer_names_list[visi_loop],False)  
                visi_loop += 1
				
                
            pdb.gimp_item_set_visible(layer_names_list[first_letter_layer],True)
            pdb.gimp_image_set_active_layer(image, layer_names_list[first_letter_layer])
				
        else:
            message_text = "There appears to be no suitable layer to create new letters from."
            old_handler = pdb.gimp_message_get_handler()	
            pdb.gimp_message_set_handler(MESSAGE_BOX)
            pdb.gimp_message(message_text)
            pdb.gimp_message_set_handler(old_handler)
		
            
		

    if do_sort:
        space_case = [' ','space']
        upper_case = ['Z','Y','X','W','V','U','T','S','R','Q','P','O','N','M','L','K','J','I','H','G','F','E','D','C','B','A']
        lower_case = ['z','y','x','w','v','u','t','s','r','q','p','o','n','m','l','k','j','i','h','g','f','e','d','c','b','a']
        numerals = ['9','8','7','6','5','4','3','2','1','0']
        mathematical = ['-','+','=','/','*']
        punctuation = ['.',',',':',';','!','?','\'','\"']
        other_characters = ['$','%','^','&','_','{','}','@','#','~','|','\\','<','>','(',')','[',']','{','}']
        special_characters = unichr(163) + unichr(169) + unichr(174) # Pound Sterling sign, Copyright Sign, Registered Trademark sign
		
        num_layers, layer_ids = pdb.gimp_image_get_layers(image)
        layer_id_list = image.layers
        loop = 0
        while loop < num_layers:
            layer_name = pdb.gimp_item_get_name(layer_id_list[loop])
            if loop == 0:
                layer_name_list = [layer_name]
            else:
                layer_name_list.append(layer_name)
            loop += 1

        all_layers = layer_name_list
		
        for character in special_characters:
            layer_test = character + ".png"
            if layer_test in all_layers:
                layer_id = pdb.gimp_image_get_layer_by_name(image,layer_test)
                pdb.gimp_image_raise_item_to_top(image,layer_id)
								
        for character in other_characters:
            layer_test = character + ".png"
            if layer_test in all_layers:
                layer_id = pdb.gimp_image_get_layer_by_name(image,layer_test)
                pdb.gimp_image_raise_item_to_top(image,layer_id)
								
        for character in punctuation:
            layer_test = character + ".png"
            if layer_test in all_layers:
                layer_id = pdb.gimp_image_get_layer_by_name(image,layer_test)
                pdb.gimp_image_raise_item_to_top(image,layer_id)
						
        for character in mathematical:
            layer_test = character + ".png"
            if layer_test in all_layers:
                layer_id = pdb.gimp_image_get_layer_by_name(image,layer_test)
                pdb.gimp_image_raise_item_to_top(image,layer_id)
				
        for character in numerals:
            layer_test = character + ".png"
            if layer_test in all_layers:
                layer_id = pdb.gimp_image_get_layer_by_name(image,layer_test)
                pdb.gimp_image_raise_item_to_top(image,layer_id)
				
        for character in lower_case:
            layer_test = character + ".png"
            if layer_test in all_layers:
                layer_id = pdb.gimp_image_get_layer_by_name(image,layer_test)
                pdb.gimp_image_raise_item_to_top(image,layer_id)
				
        for character in upper_case:
            layer_test = character + ".png"
            if layer_test in all_layers:
                layer_id = pdb.gimp_image_get_layer_by_name(image,layer_test)
                pdb.gimp_image_raise_item_to_top(image,layer_id)
			
        for character in space_case:
            layer_test = character + ".png"
            if layer_test in all_layers:
                layer_id = pdb.gimp_image_get_layer_by_name(image,layer_test)
                pdb.gimp_image_raise_item_to_top(image,layer_id)
				
    if do_layer_info:
        num_layers, layer_ids = pdb.gimp_image_get_layers(image)
        layer_list = image.layers
        loop = 0
        layer_info = ""
		
        while loop < num_layers:
            layer_name = pdb.gimp_item_get_name(layer_list[loop])
            layer_name = layer_name.decode('utf-8')
            if len(layer_name) == 5:
                layer_info += layer_name[0]
            elif (layer_name == "space.png"):
                layer_info += "space"
		
            loop += 1
        layer_info = layer_info.decode('utf-8') # Allows unicode characters above 127 to be displayed without creating errors
        old_handler = pdb.gimp_message_get_handler()	
        pdb.gimp_message_set_handler(MESSAGE_BOX)
        pdb.gimp_message(layer_info)
        pdb.gimp_message_set_handler(old_handler)
            

    if do_layer_names:
        num_layers, layer_ids = pdb.gimp_image_get_layers(image)
        layer_list = image.layers
        loop = 0
        while loop < num_layers:
            layer_name = pdb.gimp_item_get_name(layer_list[loop])
            layer_name = layer_name.decode('utf-8')
            if len(layer_name) == 1:
                layer_name += text_string
                pdb.gimp_item_set_name(layer_list[loop],layer_name)
            else:
                if layer_name == "space":
                    layer_name += text_string
                    pdb.gimp_item_set_name(layer_list[loop],layer_name)
            loop += 1
			
			
    if do_path_names:
        num_paths, vector_ids = pdb.gimp_image_get_vectors(image)
        vector_list = image.vectors
        loop = 0
        while loop < num_paths:
            path_name = pdb.gimp_item_get_name(vector_list[loop])
            path_name = path_name.decode('utf-8')
            if len(path_name) == 1:
                path_name += text_string
                pdb.gimp_item_set_name(vector_list[loop],path_name)
            else:
                if path_name == "space":
                    path_name += text_string
                    pdb.gimp_item_set_name(vector_list[loop],path_name)
            loop += 1
	
    # Set end undo point
    pdb.gimp_image_undo_group_end(image)

    # Restore the GIMP's settings
    gimp.context_pop()
	
def proc_trim_away(img, layer, amount):
    # set our feather settings
    our_feather = True
    our_feather_radius = 2
    #get feather settings
    feather = pdb.gimp_context_get_feather()
    feather_x, feather_y = pdb.gimp_context_get_feather_radius()
    # make sure that feather is set
    if not feather: pdb.gimp_context_set_feather(our_feather)
    # use our feather radius
    pdb.gimp_context_set_feather_radius(our_feather_radius, our_feather_radius)
    # select the contents of the layer
    pdb.gimp_image_select_item(img, 2, layer)
    # shrink the selection
    pdb.gimp_selection_shrink(img, amount)
    # invert selection
    pdb.gimp_selection_invert(img)
    # trim away pixels
    successful = pdb.gimp_edit_cut(layer)
    # deselect
    pdb.gimp_selection_none(img)
    # reset feather settings
    pdb.gimp_context_set_feather(feather)
    pdb.gimp_context_set_feather_radius(feather_x, feather_y)


register(
    "python_fu_Custom_Font_Label_Setting_v1-04",
    "Provides options for custom font layer names",
    "Provides options for custom font layer names",
    "Skinnyhouse",
    "Open source (BSD 3-clause license)",
    "2018",
    "<Image>/Filters/Custom Font Labels v1.04...",
    "*",
    [
        (PF_STRING, "word", "Label String", ".png"),
        (PF_TOGGLE, "sort_option", "Sort layers into order", 0),
        (PF_TOGGLE, "info_option", "Display current layer names in a message box", 0),
        (PF_TOGGLE, "layer_option", "Add Label String to single letter layer labels", 0),
        (PF_TOGGLE, "path_option", "Add Label String to single letter path labels", 0),
        (PF_TOGGLE, "random_bg_option", "Create new letter faces from visible layers", 0),
        (PF_TOGGLE, "remove", "Remove all non-font layers", 0),
    ],
    [],
    do_stuff)

main()

	

