#!/usr/bin/env python
#
# -------------------------------------------------------------------------------------
#
# Copyright (c) 2018 KGH
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#    - Redistributions of source code must retain the above copyright notice, this 
#    list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above copyright notice, 
#    this list of conditions and the following disclaimer in the documentation and/or 
#    other materials provided with the distribution.
#    - Neither the name of the author nor the names of its contributors may be used 
#    to endorse or promote products derived from this software without specific prior 
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
# SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
# DAMAGE.
#
# -------------------------------------------------------------------------------------
#
# This file is a basic Python plug-in for GIMP.
#
# It can be executed by selecting the menu option: 'Filters>>Custom Font Outlines v0.03'
#
# Change Log
# Version 0.01 Provision added to call Shine Filter v 0.01 or v 0.01b by Graechan and Chisel or Carve Filter version 1.3 by Rob Antonishen
# Version 0.02 Outline width set to zero will allow the filter to add shine and/or chisel to existing custom font layers without adding outlines when outline width set to zero
#              Distort will also work on all these outlines apart from the brushed outline when outline width set to zero and Distort selection spread value > 0
# Version 0.03 Trimmed pixels from letter faces to remove slight halo effect

# Current Version 0.03 

from gimpfu import *

def do_stuff(image, layer, outline_gradient, outline_pattern, outline_width, stroke_with_brush, stroke_with_gradient, stroke_with_pattern, untrim, restrict_brush_size, merge_layers, outlines_only, keep_paths, 
            distress_spread, distress_granularity, distress_smooth, use_shine, shine_azimuth, use_chisel, chisel_width, chisel_azimuth, chisel_elevation, chisel_mode_ptr, chisel_blur_ptr, chisel_before_shine):


    # Store the GIMP's settings so they can be restored
    gimp.context_push()
    # Set start undo point
    pdb.gimp_image_undo_group_start(image)
	
    use_clipboard_as_brush, do_layer_info, do_layer_names, do_path_names, do_sort = 0,0,0,0,0
	
    outline_colour = pdb.gimp_context_get_foreground()
	
    # check to see if our version of the shine plugin is available
    shine_available, shine_available_001b = False, False
    plugin_info = "shine"
    num_plugins, menu_path, num_plugins, plugin_accelerator, num_plugins, plugin_location, num_plugins, plugin_image_type, num_plugins, plugin_install_time, num_plugins, plugin_real_name = pdb.gimp_plugins_query(plugin_info)
		
    if "script-fu-shine-001b" in str(plugin_real_name): 
        shine_available_001b = True
    else:
        if num_plugins > 1:
            loop = 0
            while loop < num_plugins:
                if plugin_real_name[loop] == "script-fu-shine":
                    shine_available = True
                loop += 1
        
        elif num_plugins == 1:
            if "script-fu-shine" == str(plugin_real_name):
                shine_available = True
        
    if not (shine_available or shine_available_001b): use_shine = False
	
    # set non UI shine plugin parameters
    shine_shadow_X, shine_shadow_Y, shine_shadow_size, shine_shadow_opacity, shine_keep_selection, shine_keep_layers = 0,0,0,0,0,0
	
    
    # check to see if version of the chisel or carve plugin is available
    chisel_available = False
    plugin_info = "chisel"
    num_plugins, menu_path, num_plugins, plugin_accelerator, num_plugins, plugin_location, num_plugins, plugin_image_type, num_plugins, plugin_install_time, num_plugins, plugin_real_name = pdb.gimp_plugins_query(plugin_info)
    if "script-fu-chisel" in str(plugin_real_name): chisel_available = True
    if not chisel_available: use_chisel = False
	
    if chisel_mode_ptr == 0:
        chisel_mode = 0
    else:
        chisel_mode = 1
		
    if chisel_blur_ptr == 1:
        chisel_blur = 0
        chisel_softness = 0
        chisel_roundness = 0.0
        chisel_power = 0.0
    else:
        chisel_blur = 20
        chisel_softness = 10
        chisel_roundness = 1.0
        chisel_power = 2.0
	
    # set non UI chisel plugin parameters
    chisel_depth, chisel_location, chisel_keep_bump = 60, 0, 0

    loop_ptr = 1
    num_layers, layer_ids = pdb.gimp_image_get_layers(image)
    loop = num_layers - loop_ptr
    layer_names_list = image.layers
	
	
    first_char_layer_name = "none"
    # if the outline width has been set to zero but no chisel or shine has been set don't alter layers visibility
    if not (outline_width == 0 and use_chisel == False and use_shine == False):
        # turn off visibility of layers
        visi_loop, first_char_layer  = 0, -1
        while visi_loop < num_layers:
	        # get info about layer we are working on
            #layer_id = layer_ids[visi_loop]
            layer_name = pdb.gimp_item_get_name(layer_names_list[visi_loop])
            layer_name = layer_name.decode('utf-8')
            if len(layer_name) == 1 or len(layer_name) == 5 and ".png" in layer_name:
                if first_char_layer < 0: 
                    first_char_layer = visi_loop
                    first_char_layer_name = layer_name
                    #pdb.gimp_message("first_char_layer_name = " + str(first_char_layer_name) )
            pdb.gimp_item_set_visible(layer_names_list[visi_loop],False)
            visi_loop += 1
	
    # set value of distress_threshold
    distress_threshold = 127
	
	
    # set stroke method to paint line  or with the currently selected brush
    if stroke_with_brush:
        stroke_method = 1
    else:
        stroke_method = 0
		
    # Set various stroke options
    pdb.gimp_context_set_line_width(outline_width)
    pdb.gimp_context_set_stroke_method(stroke_method)
    pdb.gimp_context_set_foreground(outline_colour)
    pdb.gimp_context_set_sample_transparent(True)
    current_brush = pdb.gimp_context_get_brush()
	
    if stroke_method == 1:
        if restrict_brush_size:
            if outline_width > 0: pdb.gimp_context_set_brush_size(outline_width*1.75)
        if current_brush == "Clipboard Image":
            if not use_clipboard_as_brush:
                number_of_brushes, brushes_list = pdb.gimp_brushes_get_list("")
                if number_of_brushes > 2:
                    pdb.gimp_context_set_brush(brushes_list[2])
		
    # next 14 lines - possible version checker
    version = pdb.gimp_version()
    counter, pos = 0, 0
    dot = version.find(".", pos)
    major = version[pos:dot]
    pos = dot + 1
    dot = version.find(".", pos)
    minor = version[pos:dot]
    pos = dot + 1
    
    dot = (len(version))
    if pos < dot-1:
        release = version[pos:dot]
    else:
        release = version[pos]

    
	
    while loop >= 0:
	    # get info about layer we are working on
        layer_id = layer_ids[loop]
        layer_name = layer_names_list[loop]
	
        # cater for unicode characters above normal range
        display_layer_name =  pdb.gimp_item_get_name(layer_names_list[loop])
        display_layer_name = display_layer_name.decode('utf-8')	
	
        # check we have the layer id of the currently active layer
        pdb.gimp_image_set_active_layer(image, layer_name)
        layer_id = pdb.gimp_image_get_active_layer(image)

        # check to see if the current layer is a custom font character layer
        test_for_custom_layer = fn_test_for_custom_layer(image, layer_id, display_layer_name)
		
        # if the outline width has been set to zero but no chisel or shine has been set don't do it
        if outline_width == 0 and use_chisel == False and use_shine == False: test_for_custom_layer = False
		
        if outline_width == 0 and distress_spread > 0 and merge_layers:
            temp_outline_width = 20
            pdb.gimp_context_set_line_width(temp_outline_width)
        else:
            temp_outline_width = 0
		
        if test_for_custom_layer:

            # ensure we have the active layer and make it visible
            active_layer_id = pdb.gimp_image_get_active_layer(image)
            pdb.gimp_item_set_visible(active_layer_id, True)
			
            if outline_width > 0 or (outline_width == 0 and temp_outline_width == 20):
		
                # copy layer as outline layer
                outline_layer = pdb.gimp_layer_copy(layer_name, True)
		
                # insert new outline layer above the active layer
                pdb.gimp_image_insert_layer(image, outline_layer, None, -1)
		
                # name the new outline layer
                pdb.gimp_item_set_name(outline_layer, str(display_layer_name[0] + " outline") )
		
                # select the contents of the layer
                pdb.gimp_image_select_item(image, 2, outline_layer)
		
                # shrink the selection by the required amount
                if outline_width > 1 :pdb.gimp_selection_shrink(image, int(outline_width/2) - 1)
                if temp_outline_width == 20 :pdb.gimp_selection_shrink(image, int(temp_outline_width/2) - 1)
			
                # distort path if spread > 0
                if distress_spread > 0: pdb.script_fu_distress_selection(image, outline_layer, distress_threshold, distress_spread, distress_granularity, distress_smooth, True, True)
				
                # create path from current selection
                pdb.plug_in_sel2path(image, outline_layer)
		
                # get id of the newly created path
                active_vector_id = pdb.gimp_image_get_active_vectors(image)
		
                # give the path its name
                pdb.gimp_item_set_name(active_vector_id, str(display_layer_name[0] + " path") )
		
                # Deselect
                pdb.gimp_selection_none(image)
		
                # clear the contents of the layer
                pdb.gimp_drawable_edit_clear(outline_layer)
		
            if stroke_with_gradient:
                if outline_width > 0 or temp_outline_width == 20:
                    # create an outline using the current gradient
                    # Force stroke method to stroke with line tool
                    pdb.gimp_context_set_stroke_method(0)
				
                    # stroke the path (using current width/colour settings)
                    pdb.gimp_drawable_edit_stroke_item(outline_layer, active_vector_id)
				
                    # Make a selection from the outline to create a border selection
                    pdb.gimp_image_select_item(image, 2, outline_layer)

                    # fill the selection with the gradient
                    pdb.gimp_context_set_gradient(outline_gradient)
                    pdb.gimp_drawable_edit_gradient_fill(outline_layer, 0, 0, False, 1, 0, True, (outline_layer.width)/2, (outline_layer.height), (outline_layer.width)/2, 0)
				
                    # Deselect
                    pdb.gimp_selection_none(image)
				
            elif stroke_with_pattern:
                if outline_width > 0 or temp_outline_width == 20:
                    # create an outline using the current pattern			
                    # Force stroke method to stroke with line tool
                    pdb.gimp_context_set_stroke_method(0)
				
                    # stroke the path (using current width/colour settings)
                    pdb.gimp_drawable_edit_stroke_item(outline_layer, active_vector_id)
				
                    # Make a selection from the outline to create a border selection
                    pdb.gimp_image_select_item(image, 2, outline_layer)

                    # fill the selection with the gradient
                    pdb.gimp_context_set_pattern(outline_pattern)
                    pdb.gimp_drawable_edit_fill(outline_layer, 4)
				
                    # Deselect
                    pdb.gimp_selection_none(image)
                
            else:
                if outline_width > 0 or temp_outline_width == 20:
                    # create an outline using the currently selected brush or as a simple line
                    # set the gradient blend colour space ?
                    pdb.gimp_context_set_gradient_blend_color_space(0)
			
                    # stroke the path (using current width/colour settings)
                    pdb.gimp_drawable_edit_stroke_item(outline_layer, active_vector_id)
			
            if outline_width > 0 or temp_outline_width == 20:
                # turn on the visibility of the new outline layer for all types of outline
                pdb.gimp_item_set_visible(outline_layer, True)
                #dummy = hot
			
                if not keep_paths: pdb.gimp_image_remove_vectors(image,active_vector_id)
			
            if merge_layers:
                if not stroke_with_brush:
                    if (not untrim and outline_width > 0 or not untrim and temp_outline_width == 20):
                        pdb.gimp_image_select_contiguous_color(image, 2, outline_layer, 0, 0)
                        pdb.gimp_selection_grow(image, 1)
                        successful = pdb.gimp_edit_cut(active_layer_id)
                        pdb.gimp_selection_none(image)
                if outline_width > 0:
                    pdb.gimp_image_merge_down(image, outline_layer, 0)
                if temp_outline_width == 20:
                    # clear the contents of the layer
                    pdb.gimp_drawable_edit_clear(outline_layer)
                    pdb.gimp_image_merge_down(image, outline_layer, 0)
                active_layer_id = pdb.gimp_image_get_active_layer(image)
                if chisel_before_shine:
                    if use_chisel: pdb.script_fu_chisel(image, active_layer_id, chisel_width, chisel_softness, chisel_roundness, chisel_power, chisel_azimuth, chisel_elevation, chisel_depth, chisel_mode, chisel_location, chisel_blur, chisel_keep_bump)
                    active_layer_id = pdb.gimp_image_get_active_layer(image)
                    if use_chisel:
                        bevel_name = pdb.gimp_item_get_name(active_layer_id)
                        bevel_name += " bevel"
                        bevel_id = pdb.gimp_image_get_layer_by_name(image, bevel_name)
                        pdb.gimp_image_merge_down(image, bevel_id, 0)
                        active_layer_id = pdb.gimp_image_get_active_layer(image)
                        proc_trim_away(image, active_layer_id, 1)
                    if use_shine: 
                        if shine_available_001b:
                            pdb.script_fu_shine_001b(image, active_layer_id, shine_azimuth, shine_shadow_X, shine_shadow_Y, shine_shadow_size, shine_shadow_opacity, shine_keep_selection, shine_keep_layers)
                        else:
                            pdb.script_fu_shine(image, active_layer_id, shine_azimuth, shine_shadow_X, shine_shadow_Y, shine_shadow_size, shine_shadow_opacity, shine_keep_selection, shine_keep_layers)
                        active_layer_id = pdb.gimp_image_get_active_layer(image)	
                        proc_trim_away(image, active_layer_id, 1)	
                else:
                    if use_shine: 
                        if shine_available_001b:
                            pdb.script_fu_shine_001b(image, active_layer_id, shine_azimuth, shine_shadow_X, shine_shadow_Y, shine_shadow_size, shine_shadow_opacity, shine_keep_selection, shine_keep_layers)
                        else:
                            pdb.script_fu_shine(image, active_layer_id, shine_azimuth, shine_shadow_X, shine_shadow_Y, shine_shadow_size, shine_shadow_opacity, shine_keep_selection, shine_keep_layers)
                    active_layer_id = pdb.gimp_image_get_active_layer(image)
                    proc_trim_away(image, active_layer_id, 1)
                    if use_chisel: pdb.script_fu_chisel(image, active_layer_id, chisel_width, chisel_softness, chisel_roundness, chisel_power, chisel_azimuth, chisel_elevation, chisel_depth, chisel_mode, chisel_location, chisel_blur, chisel_keep_bump)
                    active_layer_id = pdb.gimp_image_get_active_layer(image)
                    if use_chisel:
                        bevel_name = pdb.gimp_item_get_name(active_layer_id)
                        bevel_name += " bevel"
                        bevel_id = pdb.gimp_image_get_layer_by_name(image, bevel_name)
                        pdb.gimp_image_merge_down(image, bevel_id, 0)
                        active_layer_id = pdb.gimp_image_get_active_layer(image)
                        proc_trim_away(image, active_layer_id, 1)
				
				
                if (num_layers - loop_ptr) > 0: pdb.gimp_item_set_visible(active_layer_id, False)
            elif outlines_only:
                if outline_width > 0:
                    pdb.gimp_drawable_edit_clear(active_layer_id)
                    pdb.gimp_image_merge_down(image, outline_layer, 0)
                    active_layer_id = pdb.gimp_image_get_active_layer(image)
                    if (num_layers - loop_ptr) > 0: pdb.gimp_item_set_visible(active_layer_id, False)
            else:
                brush_size = pdb.gimp_context_get_brush_size()
                if (num_layers - loop_ptr) > 0:
                    #if not stroke_with_brush:
                    if not untrim and outline_width > 0 or not untrim and temp_outline_width == 20:
                        pdb.gimp_image_select_contiguous_color(image, 2, outline_layer, 0, 0)
                        if stroke_with_brush: 
                            pdb.gimp_selection_grow(image, brush_size/4) 
                        else: 
                            pdb.gimp_selection_grow(image, outline_width/4)
                        successful = pdb.gimp_edit_cut(active_layer_id)
                        pdb.gimp_selection_none(image)
                    if outline_width > 0 or temp_outline_width == 20: pdb.gimp_item_set_visible(active_layer_id, False)	
                    if outline_width > 0 or temp_outline_width == 20: pdb.gimp_item_set_visible(outline_layer, False)	
                elif (num_layers - loop_ptr) == 0:
                    #if not stroke_with_brush:
                    if not untrim and outline_width > 0 or not untrim and temp_outline_width == 20:
                        pdb.gimp_image_select_contiguous_color(image, 2, outline_layer, 0, 0)
                        if stroke_with_brush: 
                            pdb.gimp_selection_grow(image, brush_size/4) 
                        else: 
                            pdb.gimp_selection_grow(image, outline_width/4)
                        successful = pdb.gimp_edit_cut(active_layer_id)
                        pdb.gimp_selection_none(image)
                loop_ptr += 1
			
            num_layers, layer_ids = pdb.gimp_image_get_layers(image)
			
        loop_ptr += 1
        loop = num_layers - loop_ptr
        

    if first_char_layer_name <> "none":
        if merge_layers or outlines_only:
            first_char_layer_id = pdb.gimp_image_get_layer_by_name(image, first_char_layer_name)
            pdb.gimp_item_set_visible(first_char_layer_id, True)
            pdb.gimp_image_set_active_layer(image,first_char_layer_id)
        else:
            first_char_layer_id = pdb.gimp_image_get_layer_by_name(image, first_char_layer_name)
            pdb.gimp_item_set_visible(first_char_layer_id, True)
            if outline_width > 0:first_char_layer_name = str(first_char_layer_name[0]) + " outline"
            if outline_width > 0:first_char_layer_id = pdb.gimp_image_get_layer_by_name(image, first_char_layer_name)
            if outline_width > 0:pdb.gimp_item_set_visible(first_char_layer_id, True)
            pdb.gimp_image_set_active_layer(image,first_char_layer_id)
	
    # Set end undo point
    pdb.gimp_image_undo_group_end(image)

    # Restore the GIMP's settings
    gimp.context_pop()
	
def proc_trim_away(img, layer, amount):    
    # select the contents of the layer
    pdb.gimp_image_select_item(img, 2, layer)
    # shrink the selection
    pdb.gimp_selection_shrink(img, amount)
    # invert selection
    pdb.gimp_selection_invert(img)
    # trim away pixels
    successful = pdb.gimp_edit_cut(layer)
    # deselect
    pdb.gimp_selection_none(img)
    

def fn_test_for_custom_layer(img,l_id,l_name):
    test = False
    if len(l_name) == 1:
        test = True
    elif len(l_name) == 5:
        if ".png" in l_name:
            test = True
    # select the contents of the layer
    pdb.gimp_image_select_item(img,2,l_id)
    # Test to see if the selection actually contains something
    sel_is_empty = pdb.gimp_selection_is_empty(img)
    # If there are no pixels to work with flag the layer as not viable to work on
    if sel_is_empty: test = False
    # Deselect
    pdb.gimp_selection_none(img)
    return test

register(
    "python_fu_Custom_Font_Outliner_v0-03",
    "Adds outlines to custom fonts",
    "Provides options for adding outlines to custom fonts",
    "Skinnyhouse",
    "Open source (BSD 3-clause license)",
    "2018",
    "<Image>/Filters/Custom Font Outlines v0.03...",
    "*",
    [
    #(PF_COLOR, "pf_color", "Outline colour: ", (0.0,0.0,0.0)),
    (PF_GRADIENT, "pf_current_gradient", "Gradient for outline:", 0),
    (PF_PATTERN, "pf_current_pattern", "Pattern for outline:", 0),
    (PF_SPINNER, "pf_outline_width", "Outline width in pixels: ",  10,(0,2000,1)),
    #(PF_BRUSH, "pf_brush_selector", "Brushes:", 1),
    (PF_TOGGLE, "pf_stroke_method", "Stroke outline with \ncurrent brush: ", 0),
    (PF_TOGGLE, "pf_stroke_gradient", "Stroke outline with \ngradient: ", 0),
    (PF_TOGGLE, "pf_stroke_pattern", "Stroke outline with \npattern: ", 0),
    (PF_TOGGLE, "pf_untrim", "Un-trim letter layer: ", 0),
    (PF_TOGGLE, "pf_restrict_brush", "Constrain brush size: ", 0),
    (PF_TOGGLE, "pf_merge_layers", "Merge outline with \nits letter layer: ", 0),
    (PF_TOGGLE, "pf_outline_only", "Outlines only layers: ", 0),
    (PF_TOGGLE, "pf_keep_paths", "Retain letter paths: ", 0),
    (PF_SPINNER, "pf_distress_spread", "Distort selection spread value: ", 0,(0,1000,1)),
    #(PF_SPINNER, "pf_distress_threshold", "Distort selection threshold value: ", 0,(-126,127,1)),
    (PF_SPINNER, "pf_distress_granularity", "Distort selection granularity value: ", 4,(1,25,1)),
    (PF_SPINNER, "pf_distress_smooth", "Distort selection smooth value: ", 2,(1,150,1)),
    (PF_TOGGLE, "pf_add_shine", "Use Shine when merging: ", 0),
    (PF_SPINNER, "pf_shine_azimuth", "Shine azimuth: ", 315,(0,360,1)),
    (PF_TOGGLE, "pf_add_chisel_or_carve", "Use Chisel or carve when merging: ", 0),
    (PF_SPINNER, "pf_chisel_width", "Chisel or carve bevel width: ", 20,(2,256,1)),
    (PF_SPINNER, "pf_chisel_azimuth", "Chisel or carve azimuth: ", 315,(0,360,1)),
    (PF_SPINNER, "pf_chisel_elevation", "Chisel or carve elevation: ", 45.0,(0.5,90,0.1)),
    (PF_TOGGLE, "pf_chisel_or_carve_mode", "Chisel off edges: ", 1),
    (PF_TOGGLE, "pf_hard_or_soft_edge", "Hard edged chisel: ", 1),
    (PF_TOGGLE, "pf_shine_before_chisel", "Use Chisel or Carve filter before Shine: ", 1),
    ],
    [],
    do_stuff)

main()

	

